require "test/unit"
require "phi"
class TCGrid < Test::Unit::TestCase
  def test_ancestors
    assert_equal Phi::WinControl, Phi::CustomGrid.superclass
    assert_equal Phi::CustomGrid, Phi::DrawGrid.superclass
  end

  # [Delphi] property RowHeights: Integer read GetRowHeights write SetRowHeights;
  def test_row_heights
    form = Phi::Form.new(:form)
    grid = Phi::DrawGrid.new(form)
    assert_kind_of Phi::Indexer, grid.row_heights
    assert_equal 24, grid.row_heights[0]
    grid.row_heights[0] = 12
    assert_equal 12, grid.row_heights[0]
  end

  # [Delphi] property ColWidths: Integer read GetColWidths write SetColWidths;
  def test_col_widths
    form = Phi::Form.new(:form)
    grid = Phi::DrawGrid.new(form)
    assert_kind_of Phi::Indexer, grid.col_widths
    assert_equal 64, grid.col_widths[0]
    grid.col_widths[0] = 32
    assert_equal 32, grid.col_widths[0]
  end

  # [Delphi] property TabStops: Boolean read GetTabStops write SetTabStops;
  def test_tab_stops
    form = Phi::Form.new(:form)
    grid = Phi::DrawGrid.new(form)
    assert_kind_of Phi::Indexer, grid.tab_stops
    assert grid.tab_stops[0]
    grid.tab_stops[0] = false
    assert ! grid.tab_stops[0]
  end
end

class TCValueListEditor < Test::Unit::TestCase
  def test_ancestors
    assert_equal Phi::CustomGrid, Phi::ValueListEditor.superclass
  end

  def test_new
    form = Phi::Form.new(:form)
    grid = Phi::ValueListEditor.new(form)

    assert_equal 2, grid.col_count
    assert_equal 2, grid.row_count

    assert_equal 'L[', grid.cells[0,0]
    assert_equal 'l', grid.cells[1,0]
    assert_equal '', grid.cells[0,1]
    assert_equal '', grid.cells[1,1]

    assert_equal 'L[', grid.keys[0]
    assert_equal '', grid.keys[1]

    assert_equal '', grid.values['L[']
    assert_equal '', grid.values['']

    grid.cells[0,1] = 'key1'
    grid.cells[1,1] = 'value1'
    assert_equal 'key1', grid.cells[0,1]
    assert_equal 'value1', grid.cells[1,1]
    assert_equal 'key1', grid.keys[1]
    assert_equal 'value1', grid.values['key1']

    grid.keys[1] = 'key2'
    grid.values['key2'] = 'value2'
    assert_equal 'value2', grid.values['key2']
  end
end
